/* IMPLANTATION D'UN SCHEMA PHYSIQUE ORACLE */

DROP VIEW adherent;
DROP SYNONYM adherent;
DROP TABLE adherent CASCADE CONSTRAINTS;
CREATE TABLE adherent(
numadherent NUMBER(6,0) PRIMARY KEY ,
numinstructeur NUMBER(6,0),
nomadherent VARCHAR(32) NOT NULL ,
prenomadherent VARCHAR(32) NOT NULL ,
adresseadherent VARCHAR(200) NOT NULL 
);

DROP VIEW instructeur;
DROP SYNONYM instructeur;
DROP TABLE instructeur CASCADE CONSTRAINTS;
CREATE TABLE instructeur(
numinstructeur NUMBER(6,0) PRIMARY KEY ,
numadherent NUMBER(6,0) NOT NULL UNIQUE,
salairefixe NUMBER(15,2) NOT NULL ,
CONSTRAINT instructeur_adh_fkey FOREIGN KEY(numadherent) REFERENCES adherent(numadherent)
);

ALTER TABLE adherent 
ADD CONSTRAINT adherent_ins_fkey FOREIGN KEY(numinstructeur) REFERENCES instructeur(numinstructeur)
;
/*
On peut dclarer que cette contrainte peut voir sa vrification diffre  la fin de la transaction
ALTER TABLE adherent
DROP CONSTRAINT adherent_ins_fkey
;
ALTER TABLE adherent 
ADD CONSTRAINT adherent_ins_fkey FOREIGN KEY(numinstructeur) REFERENCES instructeur(numinstructeur)
INITIALLY DEFERRED DEFERRABLE
;
ou
ALTER TABLE adherent 
ADD CONSTRAINT adherent_ins_fkey FOREIGN KEY(numinstructeur) REFERENCES instructeur(numinstructeur)
INITIALLY IMMEDIATE DEFERRABLE
;
*/

DROP VIEW type_engin;
DROP SYNONYM type_engin;
DROP TABLE type_engin CASCADE CONSTRAINTS;
CREATE TABLE type_engin(
typeavion VARCHAR(32) PRIMARY KEY ,
nbplaces NUMBER(6,0) NOT NULL ,
puissance NUMBER(6,0) NOT NULL ,
prixheuremoteur NUMBER(15,2) NOT NULL 
);

DROP VIEW avion;
DROP SYNONYM avion;
DROP TABLE avion CASCADE CONSTRAINTS;
CREATE TABLE avion(
numavion VARCHAR(32) PRIMARY KEY ,
typeavion VARCHAR(32) NOT NULL ,
anneeavion NUMBER(6,0) NOT NULL ,
hangar VARCHAR(2) NOT NULL CHECK(UPPER(hangar) IN ('H1','H2')),
datemiseenservice DATE NOT NULL ,
CONSTRAINT avion_t_fkey FOREIGN KEY(typeavion) REFERENCES type_engin(typeavion)
);

DROP VIEW est_autonome_pour;
DROP SYNONYM est_autonome_pour;
DROP TABLE est_autonome_pour CASCADE CONSTRAINTS;
CREATE TABLE est_autonome_pour(
numadherent NUMBER(6,0) NOT NULL ,
typeavion VARCHAR(32) NOT NULL ,
CONSTRAINT est_autonome_pour_pkey PRIMARY KEY(numadherent,typeavion),
CONSTRAINT est_autonome_pour_n_fkey FOREIGN KEY(numadherent) REFERENCES adherent(numadherent),
CONSTRAINT est_autonome_pour_t_fkey FOREIGN KEY(typeavion) REFERENCES type_engin(typeavion)
);

DROP VIEW est_qualifie_pour;
DROP SYNONYM est_qualifie_pour;
DROP TABLE est_qualifie_pour CASCADE CONSTRAINTS;
CREATE TABLE est_qualifie_pour(
numinstructeur NUMBER(6,0) NOT NULL ,
typeavion VARCHAR(32) NOT NULL ,
CONSTRAINT est_qualifie_pour_pkey PRIMARY KEY(numinstructeur,typeavion),
CONSTRAINT est_qualifie_pour_n_fkey FOREIGN KEY(numinstructeur) REFERENCES instructeur(numinstructeur),
CONSTRAINT est_qualifie_pour_t_fkey FOREIGN KEY(typeavion) REFERENCES type_engin(typeavion)
);

DROP VIEW vol;
DROP SYNONYM vol;
DROP TABLE vol CASCADE CONSTRAINTS;
CREATE TABLE vol(
numvol NUMBER(6,0) PRIMARY KEY ,
numinstructeur NUMBER(6,0) NULL,
numadherent NUMBER(6,0) NOT NULL ,
numavion VARCHAR(32) NOT NULL ,
datevol DATE NOT NULL ,
dureevol NUMBER(6,0) NOT NULL ,
CONSTRAINT vol_adh_fkey FOREIGN KEY(numadherent) REFERENCES adherent(numadherent),
CONSTRAINT vol_avion_fkey FOREIGN KEY(numavion) REFERENCES avion(numavion),
CONSTRAINT vol_ins_fkey FOREIGN KEY(numinstructeur) REFERENCES instructeur(numinstructeur)
);

COMMENT ON TABLE adherent IS 'Les adherents de l''aeroclub';
COMMENT ON COLUMN adherent.numadherent IS 'Numero de l''adherent';
COMMENT ON COLUMN adherent.numinstructeur IS 'Numero de l''instructeur suiveur';
COMMENT ON COLUMN adherent.nomadherent IS 'Nom de l''adherent';
COMMENT ON COLUMN adherent.prenomadherent IS 'Prenom de l''adherent';
COMMENT ON COLUMN adherent.adresseadherent IS 'Adresse de l''adherent';
COMMENT ON TABLE avion IS 'Les avions de l''aeroclub';
COMMENT ON TABLE est_autonome_pour IS 'Un adherent est autonome pour un type d''avion';
COMMENT ON COLUMN est_autonome_pour.numadherent IS 'Numero de l''adherent autonome';
COMMENT ON COLUMN est_autonome_pour.typeavion IS 'Type de l''avion';
COMMENT ON TABLE est_qualifie_pour IS 'Un instructeur est qualifie pour un type d''avion';
COMMENT ON COLUMN est_qualifie_pour.numinstructeur IS 'Numero de l''instructeur';
COMMENT ON COLUMN est_qualifie_pour.typeavion IS 'Type de l''avion';
COMMENT ON TABLE instructeur IS 'Les instructeurs de l''aeroclub';
COMMENT ON COLUMN instructeur.numinstructeur IS 'Numero de l''instructeur';
COMMENT ON COLUMN instructeur.numadherent IS 'Numero de l''adherent qui permet de decrire completement l''instructeur';
COMMENT ON COLUMN instructeur.salairefixe IS 'Salaire fixe de l''instructeur';
COMMENT ON TABLE type_engin IS 'Type des avions de l''aeroclub';
COMMENT ON COLUMN type_engin.typeavion IS 'Type d''avion';
COMMENT ON COLUMN type_engin.nbplaces IS 'Nombre de place de l''avion';
COMMENT ON COLUMN type_engin.puissance IS 'Puissance du moteur';
COMMENT ON COLUMN type_engin.prixheuremoteur IS 'Prix de l''heure du moteur';
COMMENT ON TABLE vol IS 'Les vols effectues dans l''aeroclub';
COMMENT ON COLUMN vol.numvol IS 'Numero du vol';
COMMENT ON COLUMN vol.numinstructeur IS 'Numero de l''inspecteur qui dirige le vol';
COMMENT ON COLUMN vol.numadherent IS 'Numero de l''adherent qui paye le vol';
COMMENT ON COLUMN vol.numavion IS 'Numero de l''avion';
COMMENT ON COLUMN vol.datevol IS 'Date du vol';
COMMENT ON COLUMN vol.dureevol IS 'Duree du vol';

DROP VIEW PARAMETRE;
DROP SYNONYM PARAMETRE;
DROP TABLE PARAMETRE CASCADE CONSTRAINTS;
CREATE TABLE PARAMETRE (
   ValSmig       NUMBER  NOT NULL
   ,
   NoSmig        NUMBER  DEFAULT    1
                         NOT NULL
      CONSTRAINT uqSmig
      UNIQUE
      CONSTRAINT ckConstSmig             -- garanti a tout instant
      CHECK      ( NoSmig  IN (1) )      -- l'unicite de la valeur du SMIG
   )
;
COMMENT ON TABLE PARAMETRE IS 'Table parametres';
COMMENT ON COLUMN PARAMETRE.ValSMIG IS 'Valeur actuelle du SMIG';
COMMENT ON COLUMN PARAMETRE.NoSMIG IS 'Garantie de l''unicite du SMIG';
/*
GRANT SELECT ON PARAMETRE TO PUBLIC;
*/

INSERT INTO PARAMETRE(ValSmig) VALUES(800);

ALTER TABLE adherent DISABLE CONSTRAINT adherent_ins_fkey;

INSERT INTO adherent VALUES(1,1,'DURANT','Michel','VANNES');
INSERT INTO adherent VALUES(2,2,'DUPONT','Jacques','LORIENT');
INSERT INTO adherent VALUES(3,1,'DELMAS','Christian','BREHANT');
INSERT INTO adherent VALUES(4,2,'MARTIN','Eric','PONTIVY');
INSERT INTO adherent VALUES(5,NULL,'DUBOIS','Emmanuel','VANNES');
INSERT INTO adherent VALUES(6,NULL,'DUBOIS','Michel','VANNES');
 
INSERT INTO instructeur VALUES(1,5,12000);
INSERT INTO instructeur VALUES(2,6,4000);

ALTER TABLE adherent ENABLE CONSTRAINT adherent_ins_fkey;
/*
Si la contrainte adherent_ins_fkey est INITIALLY DEFERRED DEFERRABLE, 
INSERT INTO adherent VALUES(1,1,'DURANT','Michel','VANNES');
... 
INSERT INTO instructeur VALUES(1,5,12000);
...
COMMIT;

Si la contrainte adherent_ins_fkey est INITIALLY IMMEDIATE DEFERRABLE, 
SET CONSTRAINTS ALL DEFERRED;
INSERT INTO adherent VALUES(1,1,'DURANT','Michel','VANNES');
... 
INSERT INTO instructeur VALUES(1,5,12000);
...
COMMIT;

*/

INSERT INTO type_engin VALUES('Cessna 152',2,118,495);
INSERT INTO type_engin VALUES('Piper Archer',4,180,800);
INSERT INTO type_engin VALUES('Piper Arrow',4,200,1100);
INSERT INTO type_engin VALUES('Piper Cadet PA28',4,160,700);
INSERT INTO type_engin VALUES('Robin DR 400',2,120,600);
INSERT INTO type_engin VALUES('Robin MAJOR',4,160,700);

/*
On peut viter la fonction TO_DATE par :
ALTER SESSION SET NLS_DATE_FORMAT='DD-MM-YYYY';
*/
INSERT INTO avion VALUES('F BOER','Robin MAJOR',1988,'H1',TO_DATE('03 10 1990 12:00:00 AM','DD MM YYYY HH:MI:SS AM'));
INSERT INTO avion VALUES('F CHCC','Piper Archer',1991,'H2',TO_DATE('02 09 1989 12:00:00 AM','DD MM YYYY HH:MI:SS AM'));
INSERT INTO avion VALUES('F GAQO','Robin DR 400',1976,'H1',TO_DATE('02 09 1989 12:00:00 AM','DD MM YYYY HH:MI:SS AM'));
INSERT INTO avion VALUES('F GBOY','Robin DR 400',1979,'H2',TO_DATE('09 12 2000 12:00:00 AM','DD MM YYYY HH:MI:SS AM'));
INSERT INTO avion VALUES('F GBTK','Cessna 152',1980,'H1',TO_DATE('10 11 1989 12:00:00 AM','DD MM YYYY HH:MI:SS AM'));
INSERT INTO avion VALUES('F GDTZ','Robin MAJOR',1989,'H2',TO_DATE('03 08 1988 12:00:00 AM','DD MM YYYY HH:MI:SS AM'));
INSERT INTO avion VALUES('F GFIZ','Cessna 152',1979,'H1',TO_DATE('04 02 1999 12:00:00 AM','DD MM YYYY HH:MI:SS AM'));
INSERT INTO avion VALUES('F GGOB','Piper Cadet PA28',1990,'H2',TO_DATE('09 03 1989 12:00:00 AM','DD MM YYYY HH:MI:SS AM'));
INSERT INTO avion VALUES('F GGOD','Robin MAJOR',1988,'H1',TO_DATE('08 04 1989 12:00:00 AM','DD MM YYYY HH:MI:SS AM'));
INSERT INTO avion VALUES('F GGOK','Piper Cadet PA28',1990,'H1',TO_DATE('09 05 1990 12:00:00 AM','DD MM YYYY HH:MI:SS AM'));
INSERT INTO avion VALUES('F GPFA','Piper Arrow',1999,'H1',TO_DATE('10 06 1991 12:00:00 AM','DD MM YYYY HH:MI:SS AM'));
INSERT INTO avion VALUES('F PGRG','Piper Archer',1994,'H1',TO_DATE('11 02 1995 12:00:00 AM','DD MM YYYY HH:MI:SS AM'));

INSERT INTO est_autonome_pour VALUES(1,'Cessna 152');
INSERT INTO est_autonome_pour VALUES(5,'Cessna 152');
INSERT INTO est_autonome_pour VALUES(5,'Piper Archer');
INSERT INTO est_autonome_pour VALUES(5,'Piper Arrow');
INSERT INTO est_autonome_pour VALUES(5,'Piper Cadet PA28');
INSERT INTO est_autonome_pour VALUES(5,'Robin MAJOR');
INSERT INTO est_autonome_pour VALUES(6,'Robin MAJOR');
INSERT INTO est_autonome_pour VALUES(6,'Robin DR 400');
INSERT INTO est_autonome_pour VALUES(3,'Robin MAJOR');

INSERT INTO est_qualifie_pour VALUES(1,'Cessna 152');
INSERT INTO est_qualifie_pour VALUES(1,'Piper Archer');
INSERT INTO est_qualifie_pour VALUES(1,'Piper Arrow');
INSERT INTO est_qualifie_pour VALUES(1,'Piper Cadet PA28');
INSERT INTO est_qualifie_pour VALUES(2,'Robin DR 400');
INSERT INTO est_qualifie_pour VALUES(2,'Robin MAJOR');
INSERT INTO est_qualifie_pour VALUES(1,'Robin MAJOR');


INSERT INTO vol VALUES(23,1,1,'F BOER',TO_DATE('08 01 2001 12:00:00 AM','DD MM YYYY HH:MI:SS AM'),2);
INSERT INTO vol VALUES(1,1,1,'F GAQO',TO_DATE('03 01 2001 12:00:00 AM','DD MM YYYY HH:MI:SS AM'),2);
INSERT INTO vol VALUES(2,2,1,'F GBOY',TO_DATE('04 01 2001 12:00:00 AM','DD MM YYYY HH:MI:SS AM'),1);
INSERT INTO vol VALUES(3,2,1,'F GFIZ',TO_DATE('05 01 2001 12:00:00 AM','DD MM YYYY HH:MI:SS AM'),2);
INSERT INTO vol VALUES(4,1,1,'F GBTK',TO_DATE('06 01 2001 12:00:00 AM','DD MM YYYY HH:MI:SS AM'),3);
INSERT INTO vol VALUES(5,NULL,1,'F GGOD',TO_DATE('07 01 2001 12:00:00 AM','DD MM YYYY HH:MI:SS AM'),1);
INSERT INTO vol VALUES(6,2,1,'F BOER',TO_DATE('08 01 2001 12:00:00 AM','DD MM YYYY HH:MI:SS AM'),2);
INSERT INTO vol VALUES(7,2,1,'F GDTZ',TO_DATE('09 01 2001 12:00:00 AM','DD MM YYYY HH:MI:SS AM'),2);
INSERT INTO vol VALUES(8,NULL,1,'F CHCC',TO_DATE('03 12 1999 12:00:00 AM','DD MM YYYY HH:MI:SS AM'),5);
INSERT INTO vol VALUES(9,2,1,'F GGOB',TO_DATE('05 12 1999 12:00:00 AM','DD MM YYYY HH:MI:SS AM'),3);
INSERT INTO vol VALUES(10,2,1,'F GGOK',TO_DATE('06 12 1999 12:00:00 AM','DD MM YYYY HH:MI:SS AM'),2);
INSERT INTO vol VALUES(11,NULL,1,'F GPFA',TO_DATE('06 12 1999 12:00:00 AM','DD MM YYYY HH:MI:SS AM'),1);
INSERT INTO vol VALUES(12,1,1,'F PGRG',TO_DATE('07 12 1999 12:00:00 AM','DD MM YYYY HH:MI:SS AM'),3);
INSERT INTO vol VALUES(13,1,2,'F GFIZ',TO_DATE('01 02 2000 12:00:00 AM','DD MM YYYY HH:MI:SS AM'),2);
INSERT INTO vol VALUES(14,2,2,'F CHCC',TO_DATE('03 03 2000 12:00:00 AM','DD MM YYYY HH:MI:SS AM'),4);
INSERT INTO vol VALUES(15,NULL,2,'F PGRG',TO_DATE('04 03 2000 12:00:00 AM','DD MM YYYY HH:MI:SS AM'),1);
INSERT INTO vol VALUES(16,2,2,'F PGRG',TO_DATE('05 03 2000 12:00:00 AM','DD MM YYYY HH:MI:SS AM'),3);
INSERT INTO vol VALUES(17,NULL,2,'F GGOK',TO_DATE('06 03 2000 12:00:00 AM','DD MM YYYY HH:MI:SS AM'),2);
INSERT INTO vol VALUES(18,1,2,'F GBOY',TO_DATE('07 03 2000 12:00:00 AM','DD MM YYYY HH:MI:SS AM'),1);
INSERT INTO vol VALUES(19,2,3,'F BOER',TO_DATE('08 03 2000 12:00:00 AM','DD MM YYYY HH:MI:SS AM'),3);
INSERT INTO vol VALUES(20,2,3,'F BOER',TO_DATE('09 03 2000 12:00:00 AM','DD MM YYYY HH:MI:SS AM'),2);
INSERT INTO vol VALUES(21,2,2,'F GDTZ',TO_DATE('11 04 2000 12:00:00 AM','DD MM YYYY HH:MI:SS AM'),5);
INSERT INTO vol VALUES(22,2,4,'F GFIZ',TO_DATE('26 01 2001 12:00:00 AM','DD MM YYYY HH:MI:SS AM'),2);

DROP TABLE IndVol CASCADE CONSTRAINTS;
DROP VIEW IndVol;
DROP SYNONYM IndVol;
CREATE VIEW IndVol AS
SELECT VOL.NumVol,VOL.NumInstructeur,VOL.NumAdherent,VOL.NumAvion,VOL.DateVol, VOL.DureeVol,DureeVol*PrixHeureMoteur AS CAInduit,DureeVol*.8*PrixHeureMoteur AS CoutHrsSal, 0 AS SupplementSalaire, .8*DureeVol*PrixHeureMoteur AS CoutTotal,.2*DureeVol*PrixHeureMoteur AS Marge
FROM VOL,AVION,TYPE_ENGIN
WHERE VOL.NumAvion=AVION.NumAvion
AND TYPE_ENGIN.TypeAvion=AVION.TypeAVION
AND NumInstructeur IS NULL
UNION
SELECT VOL.NumVol,VOL.NumInstructeur,VOL.NumAdherent,VOL.NumAvion,VOL.DateVol,VOL.DureeVol,DureeVol*(PrixHeureMoteur+350) AS CAInduit,DureeVol*.8*PrixHeureMoteur AS CoutHrsSal, 300*DureeVol AS SupplementSalaire, .8*DureeVol*PrixHeureMoteur+300*DureeVol AS CoutTotal,.2*DureeVol*PrixHeureMoteur+50*DureeVol AS Marge
FROM VOL,AVION,TYPE_ENGIN
WHERE VOL.NumAvion=AVION.NumAvion
AND TYPE_ENGIN.TypeAvion=AVION.TypeAVION
AND NumInstructeur IS NOT NULL
;

/*
GRANT SELECT ON adherent TO PUBLIC;
GRANT SELECT ON avion TO PUBLIC;
GRANT SELECT ON est_autonome_pour TO PUBLIC;
GRANT SELECT ON est_qualifie_pour TO PUBLIC;
GRANT SELECT ON instructeur TO PUBLIC;
GRANT SELECT ON type_engin TO PUBLIC;
GRANT SELECT ON vol TO PUBLIC;
*/